#!/bin/sh

PATH=$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

BSP_INST_HOME_PATH=$(dirname $(realpath $0))
BSP_ROOTFS_PATH=$BSP_INST_HOME_PATH/rootfs
TARGET_ROOTFS_PATH=

#CONFIG_FILE=/boot/config.txt
CONFIG_FILE=$(lsblk | grep mmcblk0p1 |sed 's/^.*part //')/config.txt
BOOT_PATH=$(lsblk | grep mmcblk0p1 |sed 's/^.*part //')
BLACKLIST=/etc/modprobe.d/raspi-blacklist.conf

set_antenna() {
    local conf="$CONFIG_FILE"
    local ant2="dtparam=ant2"

    grep -q "#.*${ant2}" $conf
    if [ $? -eq 0 ];then
        sed -i "s/#.*${ant2}/${ant2}/" $conf
    else
        echo "${ant2}" >> $conf
    fi
}

config_set() {
    local key="$1"
    local value="$2"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh set $key $value $conf
}

config_clear() {
    local key="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh clear $key $conf
}

config_get() {
    local key="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh get $key $conf
}

config_overlay_set() {
    local dtbo="$1"
    local params="$2"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_set $dtbo $conf $params
}

config_overlay_clear() {
    local dtbo="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_clear $dtbo $conf
}

config_overlay_rm() {
    local dtbo="$1"
    local conf="$CONFIG_FILE"
    if grep -q "$dtbo" "$conf";then
        ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_remove $dtbo $conf
    fi
}

module_load() {
    local name="$1"

    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*$name\)/#\1/"
    sed /etc/modules -i -e "s/^#[[:space:]]*\($name\)/\1/"

    if ! grep -q "^$name" /etc/modules; then
        printf "$name\n" >> /etc/modules
    fi
}

module_load_i2c() {
    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*i2c[-_]bcm2708\)/#\1/"
    sed /etc/modules -i -e "s/^#[[:space:]]*\(i2c[-_]dev\)/\1/"
    if ! grep -q "^i2c[-_]dev" /etc/modules; then
        printf "i2c-dev\n" >> /etc/modules
    fi
}

module_load_spi() {
    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*spi[-_]bcm2708\)/#\1/"
}

do_modules() {
    if ! [ -e $BLACKLIST ]; then
        touch $BLACKLIST
    fi

    module_load_i2c
    module_load_spi
    module_load mcp251x
}

do_config_txt() {
    local hw_model="$1"

    config_set dtparam=i2c_arm on

    config_set enable_uart 1
    config_set otg_mode 1
    set_antenna
    config_overlay_set ed-sdhost

    config_overlay_set i2c-rtc pcf8563

    config_overlay_rm spi1-1cs
    config_overlay_rm ed-mcp2515-spi1-can0
    config_overlay_set spi6-1cs cs0_pin=18,cs0_spidev=disabled
    config_overlay_set ed-mcp2515 spi6-0,oscillator=16000000,interrupt=7

    config_overlay_set uart2
    config_overlay_set uart3
    config_overlay_set uart4
    config_overlay_set uart5

    config_set gpio=11 "op,dl"
}

do_install() {
    cp $BSP_ROOTFS_PATH/boot/dt-blob.bin $TARGET_ROOTFS_PATH$BOOT_PATH
    cp $BSP_ROOTFS_PATH/boot/overlays/*.dtbo $TARGET_ROOTFS_PATH$BOOT_PATH/overlays/
}

do_add_udev_rules() {
    local udev_dir=/etc/udev/rules.d
    local udev_file=50-local.rules
    local os_version=$(cat /etc/os-release | grep VERSION= | sed -e "s/.*(//; s/).*//")
    [ ${os_version} ] || echo "Read OS version Failed!"
    case $os_version in
    bullseye | buster)
        cat << EOF > ${udev_dir}/${udev_file}
KERNEL=="serial0", SYMLINK+="rs232"
KERNEL=="ttyAMA1", SYMLINK+="rs485-3"
KERNEL=="ttyAMA2", SYMLINK+="rs485-1"
KERNEL=="ttyAMA3", SYMLINK+="rs485-2"
KERNEL=="ttyAMA4", SYMLINK+="rs485-4"
EOF
        ;;
    bookworm)
        cat << EOF > ${udev_dir}/${udev_file}
KERNEL=="serial0", SYMLINK+="rs232"
KERNEL=="ttyAMA2", SYMLINK+="rs485-3"
KERNEL=="ttyAMA3", SYMLINK+="rs485-1"
KERNEL=="ttyAMA4", SYMLINK+="rs485-2"
KERNEL=="ttyAMA5", SYMLINK+="rs485-4"
EOF
        ;;
    *)
        echo "Unknown system version"
        ;;
    esac
}

CM4_SEN_HW_MODEL="$1"

echo "CM4 Sensing(ED-CM4SEN) hardware model: [$CM4_SEN_HW_MODEL] ..."

echo "do install files ..."
do_install

echo "do ${CONFIG_FILE} ..."
do_config_txt $CM4_SEN_HW_MODEL

echo "do modules ..."
do_modules

echo "add udev rules"
do_add_udev_rules

echo "done."

exit 0
